package org.fuseri.moduleexercise.exercise;

import org.fuseri.model.dto.common.Result;
import org.fuseri.model.dto.exercise.ExerciseDto;
import org.fuseri.model.dto.exercise.ExerciseCreateDto;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@SpringBootTest
@AutoConfigureMockMvc
public class ExerciseFacadeTest {

    private final ExerciseDto exerciseDto = new ExerciseDto();

    private final ExerciseCreateDto exerciseCreateDto = new ExerciseCreateDto("name","desc",2,1);

    private final Exercise exercise = new Exercise();

    @Autowired
    ExerciseFacade facade;

    @MockBean
    ExerciseService service;

    @MockBean
    ExerciseMapper mapper;



    @Test
    void create() {

        when(mapper.fromCreateDto(exerciseCreateDto)).thenReturn(exercise);
        when(service.create(exercise)).thenReturn(exercise);
        when(mapper.toDto(exercise)).thenReturn(exerciseDto);

        ExerciseDto actualDto = facade.create(exerciseCreateDto);

        assertEquals(exerciseDto, actualDto);

    }

    @Test
    void testFindById() {
        Long id = 0L;

        when(service.find(id)).thenReturn(exercise);
        when(mapper.toDto(exercise)).thenReturn(exerciseDto);

        ExerciseDto actualDto = facade.find(id);

        assertNotNull(actualDto);
        assertEquals(exerciseDto, actualDto);
    }


    @Test
    void testFindAll() {
        int page = 0;
        Pageable pageable = PageRequest.of(0, 10);
        Page<Exercise> exercisePage = new PageImpl<>(List.of(exercise), pageable, 0);
        Result<ExerciseDto> expectedPageDto = new Result<>();


        when(service.findAll(page)).thenReturn(exercisePage);
        when(mapper.toResult(exercisePage)).thenReturn(expectedPageDto);

        Result<ExerciseDto> actualPageDto = facade.findAll(page);

        assertEquals(expectedPageDto, actualPageDto);
    }

    @Test
    void update() {
        Long id = 1L;
        when(mapper.fromCreateDto(exerciseCreateDto)).thenReturn(exercise);
        when(service.update(exercise)).thenReturn(exercise);
        when(mapper.toDto(exercise)).thenReturn(exerciseDto);

        ExerciseDto actualDto = facade.update(id, exerciseCreateDto);

        assertEquals(exerciseDto, actualDto);
    }

    @Test
    void testDelete() {
        Long id = 1L;
        facade.delete(id);
        verify(service).delete(id);
    }

}
