package org.fuseri.moduleexercise.exercise;

import org.fuseri.moduleexercise.question.Question;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.boot.test.autoconfigure.orm.jpa.TestEntityManager;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;


@DataJpaTest
class ExerciseRepositoryTest {

    @Autowired
    ExerciseRepository repository;

    @Autowired
    TestEntityManager entityManager;


    Exercise exercise = new Exercise("name", "desc", 2, 1L, new HashSet<>());

    Question question = new Question("text", new HashSet<>(), exercise);

    @Test
    void saveExercise() {
        Exercise saved = repository.save(exercise);

        Assertions.assertNotNull(saved);
        Assertions.assertEquals(exercise, saved);
    }

    @Test
    void findById() {
        entityManager.persist(exercise);
        entityManager.flush();

        Exercise found = repository.findById(exercise.getId()).orElse(null);

        Assertions.assertNotNull(found);
        Assertions.assertEquals(found, exercise);
    }


    @Test
    void filterPerDiffPerCourse() {
        entityManager.persist(exercise);
        entityManager.flush();

        Page<Exercise> found = repository.filterPerDifficultyPerCourse(PageRequest.of(0, 10), 1L, 2);

        Assertions.assertEquals(1, found.getTotalElements());
        Assertions.assertEquals(found.getContent().get(0), exercise);
    }

    @Test
    void testFindAllExercises() {
        Exercise exercise1 = new Exercise();

        entityManager.persist(exercise);
        entityManager.persist(exercise1);

        Page<Exercise> coursePage = repository.findAll(PageRequest.of(0, 42));

        Assertions.assertEquals(2, coursePage.getTotalElements());
        Assertions.assertEquals(coursePage.getContent(), Arrays.asList(exercise, exercise1));
    }

    @Test
    void getQuestionsEmptyQuestions() {
        entityManager.persist(exercise);

        var result = repository.getQuestions(PageRequest.of(0, 10), 1L);

        Assertions.assertEquals(0, result.getTotalElements());
    }

    @Test
    void getQuestions() {
        exercise.setQuestions(Set.of(question));
        entityManager.persist(exercise);

        var result = repository.getQuestions(PageRequest.of(0, 10), 1L);

        Assertions.assertEquals(1, result.getTotalElements());
        Assertions.assertEquals(result.getContent().get(0), question);
    }

    @Test
    void testDeleteExercise() {
        Long id = entityManager.persist(exercise).getId();
        entityManager.flush();

        repository.deleteById(id);

        Assertions.assertTrue(repository.findById(id).isEmpty());
    }


}
