package org.fuseri.moduleexercise.question;


import org.fuseri.model.dto.exercise.QuestionCreateDto;
import org.fuseri.model.dto.exercise.QuestionDto;
import org.fuseri.model.dto.exercise.QuestionUpdateDto;
import org.fuseri.moduleexercise.answer.AnswerService;
import org.fuseri.moduleexercise.exercise.Exercise;
import org.fuseri.moduleexercise.exercise.ExerciseService;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@SpringBootTest
public class QuestionFacadeTest {
    private final QuestionDto questionDto = new QuestionDto();

    private final QuestionCreateDto questionCreateDto = new QuestionCreateDto("name",1L,new ArrayList<>());

    private final QuestionUpdateDto questionUpdateDto = QuestionUpdateDto.builder().build();

    private final Exercise exercise = new Exercise();
    private final Question question = new Question("text",new HashSet<>(),exercise);

    @MockBean
    QuestionRepository repository;
    @Autowired
    QuestionFacade facade;

    @MockBean
    QuestionService service;

    @MockBean
    AnswerService answerService;

    @MockBean
    QuestionMapper mapper;


    @MockBean
    ExerciseService exerciseService;
//
//    @MockBean
//    QuestionMapper mapper;

    @Test
    void create() {
        long id = 1;
        when(exerciseService.find(id)).thenReturn(exercise);
        when(mapper.fromCreateDto(questionCreateDto)).thenReturn(question);
        when(service.create(question)).thenReturn(question);
        when(mapper.toDto(question)).thenReturn(questionDto);

        QuestionDto actualDto = facade.create(questionCreateDto);

        assertEquals(questionDto, actualDto);

    }

    @Test
    void testFindById() {
        Long id = 0L;

        when(service.find(id)).thenReturn(question);
        when(mapper.toDto(question)).thenReturn(questionDto);

        QuestionDto actualDto = facade.find(id);

        assertNotNull(actualDto);
        assertEquals(questionDto, actualDto);
    }




    @Test
    void update() {
        Long id = 1L;
        when(repository.existsById(id)).thenReturn(true);
//        when(repository.find)
        when(service.find(id)).thenReturn(question);
        when(mapper.fromCreateDto(questionCreateDto)).thenReturn(question);
        when(service.update(question)).thenReturn(question);
        when(mapper.toDto(question)).thenReturn(questionDto);
        when(mapper.fromUpdateDto(questionUpdateDto)).thenReturn(question);
        when(answerService.findAllByQuestionId(id)).thenReturn(Collections.emptyList());
        QuestionDto actualDto = facade.update(id,questionUpdateDto);


        assertEquals(questionDto, actualDto);
    }

    @Test
    void testDelete() {
        Long id = 1L;
        when(service.find(id)).thenReturn(question);

        facade.delete(id);
        verify(service).delete(id);
    }

}
